@@ -209,6 +209,7 @@ static void          clear_cache     (GConfSource* source);
 static void          blow_away_locks (const char *address);
 
 static GConfBackendVTable xml_vtable = {
+  sizeof (GConfBackendVTable),
   x_shutdown,
   resolve_address,
   lock,
@@ -227,7 +228,10 @@ static GConfBackendVTable xml_vtable = {
   sync_all,
   destroy_source,
   clear_cache,
-  blow_away_locks
+  blow_away_locks,
+  NULL, /* set_notify_func */
+  NULL, /* add_listener    */
+  NULL  /* remove_listener */
 };
 
 static void          
@@ -309,6 +313,7 @@ get_lock_dir_from_root_dir (const char *root_dir)
 static GConfSource*  
 resolve_address (const gchar* address, GError** err)
 {
+  struct stat statbuf;
   gchar* root_dir;
   XMLSource* xsource;
   GConfSource* source;
@@ -324,27 +329,22 @@ resolve_address (const gchar* address, GError** err)
   if (root_dir == NULL)
     return NULL;
 
-  if (mkdir(root_dir, dir_mode) < 0)
+  if (stat (root_dir, &statbuf) == 0)
     {
-      if (errno != EEXIST)
-        {
-          gconf_set_error(err, GCONF_ERROR_FAILED,
-                          _("Could not make directory `%s': %s"),
-                          (gchar*)root_dir, strerror(errno));
-          g_free(root_dir);
-          return NULL;
-        }
-      else
-        {
-          /* Already exists, base our dir_mode on it */
-          struct stat statbuf;
-          if (stat(root_dir, &statbuf) == 0)
-            {
-              dir_mode = _gconf_mode_t_to_mode (statbuf.st_mode);
-              /* dir_mode without search bits */
-              file_mode = dir_mode & (~0111); 
-            }
-        }
+      /* Already exists, base our dir_mode on it */
+      dir_mode = _gconf_mode_t_to_mode (statbuf.st_mode);
+
+      /* dir_mode without search bits */
+      file_mode = dir_mode & (~0111); 
+    }
+  else if (mkdir (root_dir, dir_mode) < 0)
+    {
+      /* Error out even on EEXIST - shouldn't happen anyway */
+      gconf_set_error (err, GCONF_ERROR_FAILED,
+		       _("Could not make directory `%s': %s"),
+		      (gchar *)root_dir, g_strerror (errno));
+      g_free (root_dir);
+      return NULL;
     }
 
   force_readonly = FALSE;
@@ -803,7 +803,13 @@ blow_away_locks (const char *address)
 
 /* Initializer */
 
-G_MODULE_EXPORT const gchar*
+#ifndef G_OS_WIN32
+/* If we use G_MODULE_EXPORT, *only* thusly marked functions will be
+ * exported, and xml-test uses other ones, too.
+ */
+G_MODULE_EXPORT
+#endif
+const gchar*
 g_module_check_init (GModule *module)
 {
   gconf_log(GCL_DEBUG, _("Initializing XML backend module"));
@@ -814,7 +820,10 @@ g_module_check_init (GModule *module)
   return NULL;
 }
 
-G_MODULE_EXPORT GConfBackendVTable* 
+#ifndef G_OS_WIN32
+G_MODULE_EXPORT
+#endif
+GConfBackendVTable* 
 gconf_backend_get_vtable(void)
 {
   return &xml_vtable;
