@@ -31,15 +31,7 @@
 #include <fcntl.h>
 
 struct poptOption options[] = {
-  { 
-    NULL, 
-    '\0', 
-    POPT_ARG_INCLUDE_TABLE, 
-    poptHelpOptions,
-    0, 
-    N_("Help options"), 
-    NULL 
-  },
+  POPT_AUTOHELP
   {
     NULL,
     '\0',
@@ -77,6 +69,7 @@ main (int argc, char** argv)
                   poptBadOption(ctx, 0),
                   poptStrerror(nextopt),
                   argv[0]);
+      poptFreeContext (ctx);
       return 1;
     }
 
@@ -97,6 +90,7 @@ main (int argc, char** argv)
   return 0;
 }
 
+#ifdef F_SETLK
 /* Your basic Stevens cut-and-paste */
 static int
 lock_reg (int fd, int cmd, int type, off_t offset, int whence, off_t len)
@@ -110,11 +104,18 @@ lock_reg (int fd, int cmd, int type, off_t offset, int whence, off_t len)
 
   return fcntl (fd, cmd, &lock);
 }
+#endif
 
+#ifdef F_SETLK
 #define lock_entire_file(fd) \
   lock_reg ((fd), F_SETLK, F_WRLCK, 0, SEEK_SET, 0)
 #define unlock_entire_file(fd) \
   lock_reg ((fd), F_SETLK, F_UNLCK, 0, SEEK_SET, 0)
+#else
+#warning Please implement proper locking
+#define lock_entire_file(fd) 0
+#define unlock_entire_file(fd) 0
+#endif
 
 static gboolean
 check_file_locking (void)
@@ -317,14 +318,15 @@ offer_delete_locks (void)
   const char *question;
 
   delete_locks = FALSE;
-  question = _("Your preferences files are currently in use. "
-               "(If you are logged in to this same account from "
-               "another computer, the other login session is probably using "
-               "your preferences files.) "
-               "You can choose to continue, but be aware that other login "
-               "sessions may become temporarily confused. "
-               "If you are not logged in elsewhere, it should be harmless to "
-               "continue.");
+  question = _("The files that contain your preference settings are "
+               "currently in use.\n\n"
+               "You might be logged in to a session from another computer, "
+               "and the other login session is using your preference "
+               "settings files.\n\n"
+               "You can continue to use the current session, but this "
+               "might cause temporary problems with the preference "
+               "settings in the other session.\n\n" 
+               "Do you want to continue?");
       
   if (ensure_gtk ())
     {
@@ -337,9 +339,9 @@ offer_delete_locks (void)
                                   "%s", question);
 
       gtk_dialog_add_buttons (GTK_DIALOG (d),
-                              GTK_STOCK_CANCEL,
+                              _("_Log Out"),
                               GTK_RESPONSE_REJECT,
-                              _("Continue"),
+                              _("_Continue"),
                               GTK_RESPONSE_ACCEPT,
                               NULL);
       
@@ -412,7 +414,7 @@ ensure_gtk (void)
   
   if (!done_init)
     {
-      ok = gtk_init_check (0, NULL);
+      ok = gtk_init_check (NULL, NULL);
       done_init = TRUE;
     }
   
